/*===================================================================================
 File: bflike1-interface.cpp. Version 4.2.1.1. Dec 5th, 2008.

 This module is a simple command line utility designed to illustrate the basic machinery 
 for accessing a Pico Card through the PicoXface class.

 Exercise:
 1. Open the Pico Card in read only mode.
=====================================================================================*/

#include "stdafx.h"
#include <string.h>
#include <stdafx.h>
#include <stdio.h>
#include <time.h>

#define pcadd(a, b, mask) \
	((((a) ^ (b)) + (((a) & (b) & (mask)) << 1)) & 0xff)

#define Nrounds 2

typedef unsigned char word;

static word s[2][16];
static word L;
static word R;

static unsigned int init_s[2][16] = {
	{
		0xd1310ba6, 0x98dfb5ac, 0x2ffd72db, 0xd01adfb7,
		0xb8e1afed, 0x6a267e96, 0xba7c9045, 0xf12c7f99,
		0x24a19947, 0xb3916cf7, 0x0801f2e2, 0x858efc16,
		0x636920d8, 0x71574e69, 0xa458fea3, 0xf4933d7e
	}, {
		0x4b7a70e9, 0xb5b32944, 0xdb75092e, 0xc4192623,
		0xad6ea6b0, 0x49a7df7d, 0x9cee60b8, 0x8fedb266,
		0xecaa8c71, 0x699a17ff, 0x5664526c, 0xc2b19ee1,
		0x193602a5, 0x75094c29, 0xa0591340, 0xe4183a3e
	}
};

static void init(void)
{
	int i, j;

	for (i = 0; i < 2; i++)
		for (j = 0; j < 16; j++)
			s[i][j] = init_s[i][j] & 0xff;
}

static void slow_key_setup(unsigned int n)
{
       
       word *p;
	   L = 0;
	   R = 0;

       do {
               p = &s[0][0];
               do {
                       int i;
                       for (i = 0; i < Nrounds / 2; i++) {
                               R ^= pcadd(s[0][L & 0xf], s[1][L >> 4], 0x55);
                               L ^= pcadd(s[0][R & 0xf], s[1][R >> 4], 0xaa);
                       }
                       *p++ ^= L;
                       *p++ ^= R;
               } while (p < &s[1][15]);
       } while (--n);

}

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
   {int           erC;
    char          erBuf[100],      //for output error message from CreatePicoXface()
                 *erParamP="";     //for error messages
    cGString      cs;              //string class compatible with Windows and Linux.
    PicoXface    *picoXfaceP;

    if ((picoXfaceP=CreatePicoXface(NULL, erBuf, sizeof(erBuf), &erC)) == NULL) {erParamP = erBuf; goto xit;}

    //Get and print version information.
    cs = picoXfaceP->FormatVersion();
    printf("%s\n", (const char*)cs);

    erC   = 0;
xit:if (erC) 
       {if (picoXfaceP) printf("%s\n", (const char*)picoXfaceP->InterpretError(picoXfaceP->Errorl(erC, "", erParamP), erBuf, sizeof(erBuf))); 
        else            printf("%s\n", erParamP);
       }

	int output;

	/*I don't know why, but it need to perform a first dummy read...*/
	do{
		if(picoXfaceP->ReadDevice(0,&output,4)< 0){
		 printf("DUMMY READING ERROR. ABORTING\n");
		 exit(EXIT_FAILURE);
		}
	}while(output==0x100);

	clock_t start, end;

	int i;
	unsigned niters[10] = {1022, 650, 700, 832, 915, 1023, 1020, 988, 799, 1002};
	
	start = clock();

	for(i = 0; i < 10; i++) {
		printf("Test %d\n",i);
		unsigned char LHD,RHD;
		unsigned char sHD[2][16];
		/*Send iteration number 'n' */
		if(picoXfaceP->WriteDevice(0,&niters[i%10],4) < 0){
			 printf("WRITING ERROR. ABORTING\n");
			 exit(EXIT_FAILURE);
			}

		/*Read L from Hardware*/
        do{
			if(picoXfaceP->ReadDevice(0,&output,4)< 0){
			 printf("L READING ERROR. ABORTING\n");
			 exit(EXIT_FAILURE);
			}
		}while(output==0x100);
		 LHD = output & 255;
		
		/*Read R from Hardware*/
        do{
			if(picoXfaceP->ReadDevice(0,&output,4)< 0){
			 printf("R READING ERROR. ABORTING\n");
			 exit(EXIT_FAILURE);
			}
		}while(output==0x100);
		RHD = output & 255;
		
		/*Read S from Hardware*/
		int xSHD,ySHD;
		for(xSHD = 0; xSHD < 2;xSHD++){
			for(ySHD = 0; ySHD < 16;ySHD++){
				do{
					if(picoXfaceP->ReadDevice(0,&saida,4)< 0){
						printf("S READING ERROR. ABORTING\n");
						exit(EXIT_FAILURE);
					}
				}while(output==0x100);
				sHD[xSHD][ySHD] = output & 255;
			}
		}
		/*Executing high level function to compare results*/
		init();
        slow_key_setup(niters[i%10]);
		
		/*Comparing values*/
		if(L != LHD)
			printf("L =%d != LHD =%d\n",L,LHD );
		
		if(R != RHD)
			printf("R =%d != RHD =%d\n",R,RHD );
		
		
		for(xSHD = 0; xSHD < 2;xSHD++){
			for(ySHD = 0; ySHD < 16;ySHD++){
				if(sHD[xSHD][ySHD] != s[xSHD][ySHD])
				 printf("sHD[%d][%d]= %d != s[%d][%d]= %d\n",xSHD,ySHD,sHD[xSHD][ySHD],xSHD,ySHD,s[xSHD][ySHD]);
			}
		}
		
	}

	end = clock();

    DeletePicoXface(picoXfaceP);

	printf("Time required for execution: %f seconds.\n",(double)(end-start)/CLOCKS_PER_SEC);

	system("pause");
    return 0;
   } //main...
