`timescale 1ns / 1ps

`define NROUNDS_DIVIDED_BY_2 1
`define PCADD(a, b, mask) (((a) ^ (b)) + (((a) & (b) & (mask)) << 1))
//`define PCADD(a, b, mask) ((a)^(b)^(mask))
//`define PCADD(a, b, mask) ((a)^(b))
//`define PCADD(a, b, mask) ((a)+(b))

module slow_key_setup1(
	input PicoClk,
	input PicoRst,
	input [31:0] PicoAddr,
	input PicoRd,
	input PicoWr,
	input [31:0] PicoDataIn,
	output reg [31:0] PicoDataOut
);

reg [7:0] s [31:0];
reg [7:0] l;
reg [7:0] r;

reg [2:0] state;

reg [4:0] sindex;
reg [9:0] nreg;

always @ (posedge PicoClk) begin
	if (PicoRst) begin
		s[0] <= 8'ha6;
		s[1] <= 8'hac;
		s[2] <= 8'hdb;
		s[3] <= 8'hb7;
		s[4] <= 8'hed;
		s[5] <= 8'h96;
		s[6] <= 8'h45;
		s[7] <= 8'h99;
		s[8] <= 8'h47;
		s[9] <= 8'hf7;
		s[10] <= 8'he2;
		s[11] <= 8'h16;
		s[12] <= 8'hd8;
		s[13] <= 8'h69;
		s[14] <= 8'ha3;
		s[15] <= 8'h7e;
		s[16] <= 8'he9;
		s[17] <= 8'h44;
		s[18] <= 8'h2e;
		s[19] <= 8'h23;
		s[20] <= 8'hb0;
		s[21] <= 8'h7d;
		s[22] <= 8'hb8;
		s[23] <= 8'h66;
		s[24] <= 8'h71;
		s[25] <= 8'hff;
		s[26] <= 8'h6c;
		s[27] <= 8'he1;
		s[28] <= 8'ha5;
		s[29] <= 8'h29;
		s[30] <= 8'h40;
		s[31] <= 8'h3e;
		l <= 8'd0;
		r <= 8'd0;
		sindex <= 0;
		nreg <= 0;
		state <= 0;
	end
   else begin
		if(PicoWr) begin
			if(state == 0) begin
				//input nreg
				nreg <= PicoDataIn[9:0];
				state <= 1;
			end
		end
		if(state == 1) begin: main_point
			//proper calculation
			integer i;
			if(nreg == 0)
				state <= 2;
			else begin
				for(i = 0; i < `NROUNDS_DIVIDED_BY_2; i=i+1) begin
					r = r ^ `PCADD(s[l[3:0]],s[16+(l>>4)], 8'h55);
					l = l ^ `PCADD(s[r[3:0]],s[16+(r>>4)], 8'haa);
				end
				s[sindex] <= s[sindex] ^ l;
				s[sindex+1] <= s[sindex+1] ^ r;
				if(sindex == 5'd30) begin
					nreg <= nreg - 1;
					sindex <= 0;
				end
				else
					sindex <= sindex + 2;
			end
		end: main_point
		if(PicoRd) begin
			if(state == 2) begin
				//output l
				PicoDataOut <= {24'd0,l};
				state <= 3;
			end
			else if(state == 3) begin
				//output r
				PicoDataOut <= {24'd0,r};
				state <= 4;
			end
			else if(state == 4) begin
				//output s
				PicoDataOut <= {24'd0,s[sindex]};
				if(sindex == 31) begin
					s[0] <= 8'ha6;
					s[1] <= 8'hac;
					s[2] <= 8'hdb;
					s[3] <= 8'hb7;
					s[4] <= 8'hed;
					s[5] <= 8'h96;
					s[6] <= 8'h45;
					s[7] <= 8'h99;
					s[8] <= 8'h47;
					s[9] <= 8'hf7;
					s[10] <= 8'he2;
					s[11] <= 8'h16;
					s[12] <= 8'hd8;
					s[13] <= 8'h69;
					s[14] <= 8'ha3;
					s[15] <= 8'h7e;
					s[16] <= 8'he9;
					s[17] <= 8'h44;
					s[18] <= 8'h2e;
					s[19] <= 8'h23;
					s[20] <= 8'hb0;
					s[21] <= 8'h7d;
					s[22] <= 8'hb8;
					s[23] <= 8'h66;
					s[24] <= 8'h71;
					s[25] <= 8'hff;
					s[26] <= 8'h6c;
					s[27] <= 8'he1;
					s[28] <= 8'ha5;
					s[29] <= 8'h29;
					s[30] <= 8'h40;
					s[31] <= 8'h3e;
					l <= 8'd0;
					r <= 8'd0;
					sindex <= 0;
					nreg <= 0;
					state <= 0;
				end
				else
					sindex <= sindex + 1;
			end
			else
				PicoDataOut <= 32'h00000100;
		end
	end
end

endmodule
